#####################################################
# gdiplib.pm
#
# These are GnuDIP common subroutines.
#
# See COPYING for licensing information.
#
# Written by:
#
#   Creighton MacDonnell
#   http://macdonnell.ca/
#
#####################################################

# Perl modules
use strict;
use Socket;
use Text::ParseWords;

# global variables
use vars qw($conf $gnudipdir);

# try for compiled MD5, otherwise use pure Perl
BEGIN {
  eval {
    require Digest::MD5;
    import  Digest::MD5 qw(md5_hex md5_base64)
  };
  if ($@) { # no Digest::MD5
    require Digest::Perl::MD5;
    import  Digest::Perl::MD5 qw(md5_hex md5_base64)
  }             
}

#####################################################
# make 10 character random salt
#####################################################
sub randomsalt {
  my @chars = ( 'A' .. 'Z', 'a' .. 'z', 0 .. 9);
  my $salt = '';
  for (my $charcount = 0; $charcount < 10; $charcount++) {
    $salt .= $chars[ rand @chars ];
  }
  return $salt;
}

#######################################################################
# strip leading blank line from string
#######################################################################
sub tst {
  my $str = shift;
  $str =~ s /^\n//;
  return $str;
}

#######################################################################
# print string with leading blank line removed
#######################################################################
sub tpr {
  print tst(shift);
}

#####################################################
# get parms from config file 
#####################################################
sub getconf {
  my $conffile = shift;
  $conffile = $gnudipdir . '/etc/gnudip.conf'
    if !defined $conffile;
  my %result;
  local *CONF;
  if (! open (CONF, "<$conffile")) {
    print STDERR "getconf: cannot open $conffile\n";
    print STDERR "getconf: $!\n";
    print STDERR 'getconf: effective user - ' . getpwuid($>) . "\n";
    my $grps = '';
    foreach my $gid ($)) {
      $grps .= ' ' .  getgrgid($gid);
    }
    print STDERR 'getconf: effective groups -' . $grps . "\n";
    return undef;
  }
  while (<CONF>) {
    my $line = $_;
    if ($line =~ /^(.*?)\#(.*)$/) {
      $line = $1;
    }    
    if ($line =~ /^\s*(\w[-\w|\.]*)\s*=\s*(.*?)\s*$/) {
      if ($result{$1}) {
        $result{$1} .= ' ' . $2;
      } else {
        $result{$1} = $2;
      }
    }
  }
  close CONF;
  return \%result;
}

#####################################################
# call logger command
#####################################################
sub calllogger {

  # how callcommand can display errors
  my $printerror = sub {
    my $line = shift;
    print STDERR $line . "\n";
    };

  return callcommand($printerror, @_);
}

#####################################################
# call BIND nsupdate command
# $conf should be a global variable
#####################################################
sub callnsupdate {
  my $domain = shift;
  if (!$domain) {
    writelog('callnsupdate: domain not passed');
    return '';
  }

  if (!$conf) {
    writelog('callnsupdate: $conf is not defined');
    return '';
  }
  if (!$conf) {
    writelog('callnsupdate: $$conf{\'nsupdate\'} is not defined');
    return '';
  }

  # BIND nsupdate command for our situation
  my $nsupdate = $$conf{'nsupdate'};
  $nsupdate = $$conf{"nsupdate.$domain"}
           if $$conf{"nsupdate.$domain"};
  return callcommand(\&writelog, $nsupdate, @_, '');
}

#####################################################
# call sendmail command
# $conf should be a global variable
#####################################################
sub callsendmail {
  my $email = shift;
  if (!$email) {
    writelog('callsendmail: no email was passed');
    return '';
  }

  if (!$conf) {
    writelog('callnsupdate: $conf is not defined');
    return '';
  }
  if (!$conf) {
    writelog('callnsupdate: $$conf{\'sendmail\'} is not defined');
    return '';
  }

  return callcommand(\&writelog, $$conf{'sendmail'}, $email);
}

#####################################################
# call a command that reads standard input
#####################################################
sub callcommand {

  my $writelog = shift;
  if (!$writelog) {
    print STDERR 'callcommand: writelog subroutine not passed';
    return '';
  }

  my $command = shift;
  if (!$command) {
    &$writelog('callcommand: command line not passed');
    return '';
  }

  # collect arguments as input text
  my @text;
  foreach my $line (@_) {
    if ($line =~ /\n/) {
      # split on new line
      push @text, (split(/\n/, $line));
    } else {
      push @text, ($line);
    }
  }

  # for trouble shooting
  #&$writelog($command);
  #foreach my $line (@text) {
  #  &$writelog($line);
  #}

  # (local) pipe for retrieving output from command
  local *RESPREAD;
  local *RESPWRITE;
  pipe(RESPREAD, RESPWRITE);

  # flush before fork
  select(STDERR);
  $| = 1;
  select(STDOUT);
  $| = 1;

  # writable open with a fork to run command
  local *COMMAND;
  my $pid = open(COMMAND, "|-");
  if (! defined $pid) {
    # fork failed
    # message to log
    &$writelog('callcommand: open failed');
    return '';
  }
  if ($pid eq 0) {
    # child
    # direct all output to response pipe
    open (STDOUT, ">&RESPWRITE") or
      &$writelog('callcommand: cannot redirect STDOUT');
    open (STDERR, ">&STDOUT") or
      &$writelog('callcommand: cannot redirect STDERR');
    # pass control to command
    exec(shellwords($command)) or exit 9999;
  }
  # parent

  # feed each line of input text to command
  foreach my $line (@text) {
    print COMMAND "$line\n";
  }

  # close (and wait) for it
  my $close = close COMMAND;
  my $exitval = $?;
  close RESPWRITE;

  # close (i.e. the command) failed?
  if (!$close or $exitval ne 0) {
    &$writelog('callcommand: command failed');
    &$writelog("  command:  $command");
    # log the input text
    foreach my $line (@text) {
      &$writelog("  input:   $line");
    }
    # log the output from command
    while (my $line = <RESPREAD>) {
      chomp($line);
      &$writelog("  output:  $line");
    }
    close RESPREAD;
    return '';
  }

  close RESPREAD;
  return 1;
}

#####################################################
# check for remote address in service list
#####################################################
sub validip {

  # get IP address to check
  my $check_ip = shift;

  my $networks = $$conf{'networks'};

  # trim leading and trailing blanks
  if ($networks =~ /^\s*(.*?)\s*$/) {
    $networks = $1;
  }

  my $good_ip = 'no';
  my $bad_ip = 'no';

  # while more networks and no decision yet
  while ($networks ne '' && $good_ip eq 'no' && $bad_ip eq 'no') {

    # break off leading network
    $networks .= ' ';
    my $network;
    if ($networks =~ /^(.*?) (.*)$/) {
      $network = $1;
      $networks = $2;
    }

    # parse leading network 
    my $not = 'no';
    if ($network =~ /^\!(.*)$/) {
      $not = 'yes';
      $network = $1;
    }
    my $netmask = '255.255.255.255';
    if ($network =~ /^(.*?)\/(.*)$/) {
      $network = $1;
      $netmask = $2;
    }

    # do the check
    my $remote_ip_bin = inet_aton($check_ip);
    my $network_bin = inet_aton($network);
    my $netmask_bin = inet_aton($netmask);
    if ($network_bin && $netmask_bin) {
      if ($network_bin eq ($remote_ip_bin & $netmask_bin)) {
        if ($not eq 'yes') {
          $bad_ip = 'yes';
        } else {
          $good_ip = 'yes';
        }
      }
    }

    # trim leading and trailing blanks
    if ($networks =~ /^\s*(.*?)\s*$/) {
      $networks = $1;
    }
  }

  if ($good_ip eq 'yes') {
    return 1;
  } else {
    return '';
  }
}

#####################################################
# must return 1
#####################################################
1;

